# Tester.py
#
# by jdog5000
#
# Provides event debug output

from CvPythonExtensions import *
import sys
import Popup as PyPopup
from PyHelpers import PyPlayer
import CvEventManager
from PyHelpers import *
import CvUtil

# globals
gc = CyGlobalContext()
localText = CyTranslator()
game = CyGame()

class Tester :

    def __init__(self, customEM, config):

        CvUtil.pyPrint("Initializing Tester")

        self.LOG_DEBUG = config.getboolean("CONFIG", "DebugMode", False )
        self.EventKeyDown=6
        self.testerPopup = 8000
        self.waitingForPopup = 8001
        self.setNamePopup = 8002

        self.iPrevTurn = 0
        self.turnPlayers = list()

        if( self.LOG_DEBUG ) : customEM.addEventHandler( 'greatPersonBorn', self.onGreatPersonBorn )
        #if( self.LOG_DEBUG ) : customEM.addEventHandler( 'techAcquired', self.onTechAcquired )
        if( self.LOG_DEBUG ) : customEM.addEventHandler( 'religionFounded', self.onReligionFounded )
        if( self.LOG_DEBUG ) : customEM.addEventHandler( 'changeWar', self.onChangeWar )
        if( self.LOG_DEBUG ) : customEM.addEventHandler( 'cityLost', self.onCityLost )
        customEM.addEventHandler( 'GameStart', self.onGameStart )
        customEM.addEventHandler( 'OnLoad', self.onGameLoad )
        customEM.addEventHandler( "kbdEvent", self.onKbdEvent )
        #customEM.addEventHandler( "EndPlayerTurn", self.endTurnPopup )
        customEM.addEventHandler( "BeginPlayerTurn", self.onBeginPlayerTurn )

        customEM.setPopupHandler( self.testerPopup, ["testerPopup",self.testerPopupHandler,self.blankHandler] )
        customEM.setPopupHandler( self.waitingForPopup, ["waitingForPopup",self.waitingForHandler,self.blankHandler] )
        customEM.setPopupHandler( self.setNamePopup, ["setNamePopup",self.setNameHandler,self.blankHandler] )

        self.customEM = customEM

        self.config = config
        self.foundConfig = config.getboolean("CONFIG", "FoundConfig", False)
        self.showActivePopup = config.getboolean("CONFIG", "ActivePopup", False)

    def blankHandler( self, playerID, netUserData, popupReturn ) :
        # Dummy handler to take the second event for popup
        return

    def onGameStart( self, argsList ) :
        if( self.showActivePopup or not self.foundConfig ) :
            self.onGameLoad( None )

    def onGameLoad( self, argsList ) :

        if( not self.foundConfig ) :
            popup = PyPopup.PyPopup( )       
            bodStr = "WARNING:  Revolution.ini not found!  Revolution components using default settings."
            bodStr += "  Check mod installation directory, should be: (civ install directory)\Warlords\Mods\Revolution\."
            popup.setBodyString( bodStr )
            popup.launch()
            
        if( self.showActivePopup ) :
            bodStr = "Revolution mod running with the following components:\n"
            if( self.config.getboolean("AIAutoPlay", "Enable", True) ) :
                bodStr += "\nAIAutoPlay"
            if( self.config.getboolean("BarbarianCiv", "Enable", True) ) :
                bodStr += "\nBarbarianCiv"
            if( self.config.getboolean("Rebellion", "Enable", True) ) :
                bodStr += "\nRebellion"
            if( self.config.getboolean("ChangePlayer", "Enable", False) ) :
                bodStr += "\nChangePlayer"
            if( self.config.getboolean("Revolution", "Enable", False) ) :
                bodStr += "\nRevolution"

            bodStr += "\n\nMax civs allowed: %d"%(gc.getMAX_CIV_PLAYERS())
            bodStr += "\nTurns in game: %d"%(game.getMaxTurns())

            popup = PyPopup.PyPopup( )       
            popup.setBodyString( bodStr )
            popup.launch()
        

    def onGreatPersonBorn(self, argsList):
        'Unit Promoted'
        pUnit, iPlayer, pCity = argsList
        player = PyPlayer(iPlayer)
        if pUnit.isNone() or pCity.isNone():
                return

        CvUtil.pyPrint('A %s was born for %s in %s' %(pUnit.getName(), player.getCivilizationName(), pCity.getName()))

    
    def onTechAcquired(self, argsList):
		'Tech Acquired'
		iTechType, iTeam, iPlayer, bAnnounce = argsList
		# Note that iPlayer may be NULL (-1) and not a refer to a player object

		CvUtil.pyPrint('%s was finished by Team %d' 
			%(PyInfo.TechnologyInfo(iTechType).getDescription(), iTeam))

    def onReligionFounded(self, argsList):
		'Religion Founded'
		iReligion, iFounder = argsList
		player = PyPlayer(iFounder)
		
		CvUtil.pyPrint('Player %d Civilization %s has founded %s'
			%(iFounder, player.getCivilizationName(), gc.getReligionInfo(iReligion).getDescription()))


    def onChangeWar(self, argsList):
		'War Status Changes'
		bIsWar = argsList[0]
		iPlayer = argsList[1]
		iRivalTeam = argsList[2]
		if (bIsWar):
			strStatus = "declared war"
		else:
			strStatus = "declared peace"
		
		CvUtil.pyPrint('Player %d Civilization %s has %s on Team %d'%(iPlayer, gc.getPlayer(iPlayer).getCivilizationDescription(0), strStatus, iRivalTeam))

    def onCityLost(self, argsList):
		'City Lost'
		city = argsList[0]
		player = PyPlayer(city.getOwner())

		CvUtil.pyPrint('City %s was lost by Player %d Civilization %s' 
			%(city.getName(), player.getID(), player.getCivilizationName()))
		
    def onKbdEvent(self, argsList ):
        'keypress handler'
        eventType,key,mx,my,px,py = argsList

        if ( eventType == self.EventKeyDown ):
	    theKey=int(key)
	    
            # For debug or trial only
	    if( theKey == int(InputTypes.KB_T) and self.customEM.bShift and self.customEM.bCtrl ) :

                popup = PyPopup.PyPopup(self.testerPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL)
                popup.setHeaderString( 'Test popup' )
                bodStr = localText.getText("TXT_KEY_TEST", ())
                bodStr += localText.getText("TXT_KEY_TRAIT_CREATIVE", ())
                popup.setBodyString( bodStr )
##                buttons = tuple()
##                popup.addButton( 'First button' )
##                popup.addButton( 'Offer button' )
##                popup.addButton( 'Buyoff button' )
##                popup.setUserData( ('first','offer','buyoff') )
                popup.launch()

            if( theKey == int(InputTypes.KB_Q) and self.customEM.bShift and self.customEM.bCtrl ) :
                self.showActivePopup = True
                self.onGameLoad( [] )

##            if( theKey == int(InputTypes.KB_E) and self.customEM.bShift and self.customEM.bCtrl ) :
##                self.waitingFor()
##
##            if( theKey == int(InputTypes.KB_U) and self.customEM.bShift and self.customEM.bCtrl ) :
##                CvUtil.pyPrint('Unpausing')
##                game.setPausePlayer(-1)

            if( theKey == int(InputTypes.KB_N) and self.customEM.bShift and self.customEM.bCtrl ) :
                self.setName()

##            if( theKey == int(InputTypes.KB_C) and self.customEM.bShift and self.customEM.bCtrl ) :
##                self.printUnitCity()

    def printUnitCity( self ) :

        unit = CyInterface().getHeadSelectedUnit()
        homeCity = gc.getCity( unit.getHomeCityID() )

        mess = "%s was built in %s."%(unit.getName(),homeCity.getName())
        CyInterface().addImmediateMessage(mess,"")
        

    def testerPopupHandler( self, iPlayerID, netUserData, popupReturn ) :

        CvUtil.pyPrint('Handling tester popup')
        if( game.isPaused() ) :
            CvUtil.pyPrint('Unpausing')
            game.setPausePlayer(-1)


    def endTurnPopup( self, argsList ) :

        iGameTurn, iPlayer = argsList

        playerI = gc.getPlayer(iPlayer)

        if( playerI.isHuman() ) :
            popup = PyPopup.PyPopup(self.testerPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL)
            popup.setBodyString( "Attempting to pause" )
            #CyInterface().setPausedPopups( True )
            game.setPausePlayer(game.getActivePlayer())
            popup.launch()


    def onBeginPlayerTurn( self, argsList ) :

        iGameTurn, iPlayer = argsList

        #CvUtil.pyPrint("  Turn %d - Player %d's turn"%(iGameTurn,iPlayer))
        if( iGameTurn > self.iPrevTurn ) :
            if( iPlayer > 0 ) :
                found = False
                for i in range(0,iPlayer) :
                    if( gc.getPlayer(i).isAlive() ) :
                        CvUtil.pyPrint("  Error!  Early turn %d - Player %d's turn"%(iGameTurn,iPlayer))
                        found = True
                        break

                if( not found ) :
                    self.iPrevTurn = iGameTurn
                    self.turnPlayers = list()
                    self.turnPlayers.append(iPlayer)
                else :
                    if( iPlayer in self.turnPlayers ) :
                        CvUtil.pyPrint("  Error!  Repeated turn %d - Player %d's turn"%(iGameTurn,iPlayer))
                    else :
                        self.turnPlayers.append(iPlayer)
                
            else :
                self.iPrevTurn = iGameTurn
                self.turnPlayers = list()
                self.turnPlayers.append(iPlayer)

        else :
            if( iPlayer in self.turnPlayers ) :
                CvUtil.pyPrint("  Error!  Repeated turn %d - Player %d's turn"%(iGameTurn,iPlayer))
            else :
                self.turnPlayers.append(iPlayer)
            

    def waitingFor( self ) :

        # This is an attempt to automatically fix the 'waiting for other ...' problem

        popup = PyPopup.PyPopup(self.waitingForPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL, bDynamic = False)

        humanStr = 'Num humans alive: %d  ever: %d\nHuman civs:\n'%(game.countHumanPlayersAlive(),game.countHumanPlayersEverAlive())
        activeStr = '\nActive civs:\n'
        for i in range(0,gc.getMAX_CIV_PLAYERS()) :
            playerI = gc.getPlayer(i)
            if( playerI.isHuman() ) :
                humanStr += playerI.getCivilizationDescription(0) + '\n'

            if( playerI.isTurnActive() ) :
                activeStr += playerI.getCivilizationDescription(0) + '\n'

        bodStr = humanStr + activeStr
        if( game.isPaused() ) :
            bodStr += '\nGame is paused'
        popup.setBodyString( bodStr )

        if( game.countHumanPlayersAlive()==0 ) :
            popup.addButton( 'Set player %d to human'%(game.getActivePlayer()) )
        
        
        #popup.popup.setTimer( 10000 )
        popup.launch()

        
    def waitingForHandler( self, iPlayerID, netUserData, popupReturn ) :

        if( popupReturn.getButtonClicked() == 0 ):

            pPlayer = gc.getActivePlayer()

            game.changePlayer( pPlayer.getID(), pPlayer.getCivilizationType(), pPlayer.getLeaderType(), -1, True, False )

            self.waitingFor()

    def setName( self ) :

        popup = PyPopup.PyPopup(self.setNamePopup,contextType = EventContextTypes.EVENTCONTEXT_ALL)
        popup.setBodyString( 'Change name of a civ' )
        popup.addSeparator()
        
        popup.createPythonEditBox( 'Name', 'New name for player', 0)
        popup.createPythonEditBox( 'Desc', 'New name for civ', 1)
        popup.createPythonEditBox( 'Short', 'New name for civ', 2)
        popup.createPythonEditBox( 'Adj', 'New name for civ', 3)

        popup.createPythonPullDown( 'This AI Player', 1 )
        for i in range(0,gc.getMAX_CIV_PLAYERS()) :
            player = PyPlayer(i)
            if( not player.isNone() ) :
                if( player.isAlive() ) :
                    popup.addPullDownString( "%s of the %s"%(player.getName(),player.getCivilizationName()), i, 1 )

        popup.addButton('None')
        CvUtil.pyPrint('Launching pick human popup')
        popup.launch()

    def setNameHandler( self, iPlayerID, netUserData, popupReturn ) :

        if( popupReturn.getButtonClicked() == 0 ):
            return

        newName = popupReturn.getEditBoxString(0)
        newCivDesc  = popupReturn.getEditBoxString(1)
        newCivShort = popupReturn.getEditBoxString(2)
        newCivAdj   = popupReturn.getEditBoxString(3)

        iPlayer = popupReturn.getSelectedPullDownValue( 1 )

        pPlayer = gc.getPlayer( iPlayer )

        CvUtil.pyPrint('Name before: %s, key: %s'%(pPlayer.getName(),pPlayer.getNameKey()))
        
        newStr = newName.encode('latin-1')
        CvUtil.pyPrint('New name: %s'%(newStr))
        pPlayer.setName( newStr )
        CvUtil.pyPrint('Key after: %s, name: %s'%(pPlayer.getNameKey(),pPlayer.getName()))

        CvUtil.pyPrint('Civ before: %s, key: %s'%(pPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescriptionKey()))

        newDesc  = newCivDesc.encode('latin-1')
        newShort = newCivShort.encode('latin-1')
        newAdj   = newCivAdj.encode('latin-1')
        CvUtil.pyPrint('New name: %s'%(newDesc))
        pPlayer.setCivName( newDesc, newShort, newAdj )
        CvUtil.pyPrint('Civ after: %s, key: %s'%(pPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescriptionKey()))
